
#include <stdarg.h>


int utoi(const char *decstr, int *nbr);

char *itox(int nbr, char *str, int sz);
char *itod(int nbr, char *str, int sz);
char *itou(int nbr, char *str, int sz);


extern void cout(char ch);

/*
** printf(controlstring, arg, arg, ...) -- formatted print
**        operates as described by Kernighan & Ritchie
**        only d, x, c, s, and u specs are supported.
*/
void printf(const char *fmtstr, ...)
{
   va_list arglist;

   int  width, prec, preclen, len;
   const char *ctl, *cx;
   char c, right, str[7], *sptr, pad;
   int i;

   va_start(arglist, fmtstr);

  ctl = fmtstr;

  while((c = *ctl++))
  {
    if(c != '%')
    {
      cout(c);
      continue;
    }

    if(*ctl == '%')
    {
      cout(*ctl++);
      continue;
    }
    
    cx = ctl;

    if(*cx == '-')
    {
      right=0;
      cx++;
    }
    else right=1;
      
    if(*cx=='0')
    {
      pad = '0';
      cx++;
    } else pad = ' ';
    
    if((i = utoi(cx, &width)) >= 0) cx += i;
    else continue;

    if(*cx=='.')
    {
      if((preclen = utoi(++cx, &prec)) >= 0) cx += preclen;
      else continue;
    }
    else preclen=0;

    sptr = str;
    c = *cx++;
    i = va_arg(arglist, int);
    
    if(c=='d') itod(i, str, 7);
    else if(c=='x') itox(i, str, 7);
    else if(c=='c') {str[0] = i; str[1] = '\0';}
    else if(c=='s') sptr = (char *)i;
    else if(c=='u') itou(i, str, 7);
    else continue;
    
    ctl = cx; /* accept conversion spec */

    if(c != 's') while(*sptr==' ') ++sptr;

    len = -1;
    while(sptr[++len]); /* get length */

    if((c=='s') && (len > prec) && (preclen > 0)) len = prec;

    if(right) while(((width--)-len)>0) cout(pad);

    while(len)
    {
      cout(*sptr++);
      --len;
      --width;
    }

    while(((width--) - len) > 0) cout(pad);
  }

   va_end(arglist);
}




int utoi(const char *decstr, int *nbr)
{
  int d, t;

  d = 0;
  *nbr = 0;
  
  while((*decstr >= '0') && (*decstr <= '9'))
  {
    t = *nbr;
    t = (10 * t) + (*decstr++ - '0');
    if ((t >= 0) && (*nbr < 0)) return -1;
    d++;
    *nbr = t;
  }

  return d;
}



/*
** itox -- converts nbr to hex string of length sz
**         right adjusted and blank filled, returns str
**
**        if sz > 0 terminate with null byte
**        if sz = 0 find end of string
**        if sz < 0 use last byte for data
*/
char *itox(int nbr, char *str, int sz)
{
  int digit, offset;

  if(sz > 0)
    str[--sz] = 0;
  else if(sz < 0)
    sz = -sz;
  else
    while(str[sz] != 0) ++sz;

  while(sz)
  {
    digit = nbr & 15;
    nbr = (nbr >> 4) & 4095;

    if(digit < 10)
      offset = 48;
    else
      offset = 55;
      
    str[--sz] = digit + offset;

    if(nbr == 0) break;
  }

  while(sz) str[--sz] = ' ';

  return str;
}


/*
** itod -- convert nbr to signed decimal string of width sz
**         right adjusted, blank filled; returns str
**
**        if sz > 0 terminate with null byte
**        if sz = 0 find end of string
**        if sz < 0 use last byte for data
*/
char *itod(int nbr, char *str, int sz)
{
  char sgn;
  unsigned int uval;

  if(nbr < 0)
  {
    uval = -nbr;
    sgn = '-';
  }
  else
  {
    uval = nbr;
    sgn = ' ';
  }

  if(sz > 0)
     str[--sz] = '\0';
  else if(sz < 0)
     sz = -sz;
  else
     while(str[sz] != '\0') ++sz;

  while(sz)
  {
    str[--sz] = (uval % 10) + '0';
    if((uval = uval / 10) == 0) break;
  }

  if(sz) str[--sz] = sgn;

  while(sz > 0) str[--sz] = ' ';

  return str;
}


/*
** itou -- convert nbr to unsigned decimal string of width sz
**         right adjusted, blank filled; returns str
**
**        if sz > 0 terminate with null byte
**        if sz = 0 find end of string
**        if sz < 0 use last byte for data
*/
char *itou(int nbr, char *str, int sz)
{
   int lowbit;

   if(sz > 0)
      str[--sz] = 0;
   else if(sz < 0)
      sz = -sz;
   else
      while(str[sz] != 0) ++sz;

   while(sz)
   {
      lowbit = nbr & 1;
      nbr = (nbr >> 1) & 32767;  /* divide by 2 */
      str[--sz] = ((nbr % 5) << 1) + lowbit + '0';
      if((nbr = nbr / 5)==0) break;
   }

   while(sz) str[--sz]=' ';

   return str;
}

